@@ -2,7 +2,7 @@
 #
 #  Net::Server - Extensible Perl internet server
 #
-#  $Id: Server.pm,v 1.112 2007/03/23 18:39:48 rhandom Exp $
+#  $Id: Server.pm,v 1.114 2007/07/25 16:21:14 rhandom Exp $
 #
 #  Copyright (C) 2001-2007
 #
@@ -37,7 +37,7 @@ use Net::Server::Daemonize qw(check_pid_file create_pid_file
                               safe_fork
                               );
 
-$VERSION = '0.96';
+$VERSION = '0.97';
 
 ###----------------------------------------------------------------###
 
@@ -154,6 +154,7 @@ sub run_client_connection {
 
   $self->post_process_request;      # clean up client connection, etc
 
+  $self->post_client_connection_hook; # one last hook
 }
 
 ###----------------------------------------------------------------###
@@ -909,6 +910,7 @@ sub process_request {
 ### user customizable hook
 sub post_process_request_hook {}
 
+sub post_client_connection_hook {}
 
 ### this is server type specific functions after the process
 sub post_process_request {
@@ -1196,17 +1198,26 @@ sub open_syslog {
   my $self = shift;
   my $prop = $self->{server};
 
-  my $logsock = defined($prop->{syslog_logsock})
-    ? $prop->{syslog_logsock} : 'unix';
-  $prop->{syslog_logsock} = ($logsock =~ /^(unix|inet|stream)$/)
-    ? $1 : 'unix';
+  require Sys::Syslog;
+
+  if (ref($prop->{syslog_logsock}) eq 'ARRAY') {
+    # do nothing - assume they have what they want
+  } else {
+    if (! defined $prop->{syslog_logsock}) {
+      $prop->{syslog_logsock} = ($Sys::Syslog::VERSION < 0.15) ? 'unix' : '';
+    }
+    if ($prop->{syslog_logsock} =~ /^(|native|tcp|udp|unix|inet|stream|console)$/) {
+      $prop->{syslog_logsock} = $1;
+    } else {
+      $prop->{syslog_logsock} = ($Sys::Syslog::VERSION < 0.15) ? 'unix' : '';
+    }
+  }
 
   my $ident = defined($prop->{syslog_ident})
     ? $prop->{syslog_ident} : 'net_server';
   $prop->{syslog_ident} = ($ident =~ /^([\ -~]+)$/)
     ? $1 : 'net_server';
 
-  require Sys::Syslog;
 
   my $opt = defined($prop->{syslog_logopt})
     ? $prop->{syslog_logopt} : $Sys::Syslog::VERSION ge '0.15' ? 'pid,nofatal' : 'pid';
@@ -1218,7 +1229,9 @@ sub open_syslog {
   $prop->{syslog_facility} = ($fac =~ /^((\w+)($|\|))*/)
     ? $1 : 'daemon';
 
-  Sys::Syslog::setlogsock($prop->{syslog_logsock}) || die "Syslog err [$!]";
+  if ($prop->{syslog_logsock}) {
+    Sys::Syslog::setlogsock($prop->{syslog_logsock}) || die "Syslog err [$!]";
+  }
   if( ! Sys::Syslog::openlog($prop->{syslog_ident},
                              $prop->{syslog_logopt},
                              $prop->{syslog_facility}) ){
